function [beta, s, theta, C, invC, lambda] = calibrate_kriging(Xs, Ys)
% [beta, s, theta, C] = calibrate_kriging(X,Y)
%	Computes the kriging model given a set of samples
%	X with evaluation values Y.
%
%	The kriging model is determined by beta, s, theta,
%	C, invC and lambda.
%
%	The kriging kernel function that is used is:
%
%	c(theta) = exp(-theta * |x - x'|) (2.2.7)
%
%	theta is estimated using the maximum likelihood
%	heuristic (2.4.13) - (2.4.17), by minimizing
%	the equation of (2.4.17) for theta
%
%	lambda is calculated from (2.4.20) - (2.4.23)
%
%	Note / TODO: optimization of theta should be done
%	in a better way!

	m = length(Xs(:,1));

	Cv = zeros(m,m);
	for i = 1:m,
		for j = 1:m,
			% Use a quadratic kernel to allow extrapolation
			% (mind to change this also in kriging estimate )
			Cv(i,j) = sum((Xs(i,:) - Xs(j,:)).^2);

			%Cv(i,j) = norm(Xs(i,:) - Xs(j,:));
			% Use a linear kernel to disallow extrapolation
			% (mind to change this also in kriging estimate)
		end
	end

	e_min = inf;
	i_min = NaN;
	theta_m = 1;
	for i = -50:4
		theta = 10^i;
		E = energy(theta, Cv, Ys, m);
		if (E <= e_min)
			i_min = i;
			theta_m = theta;
			e_min = E;
		end
	end

	if (isnan(i_min))
		error('Error in the calibration of the Kriging-model')
	end

	e_min = inf;
	for j = 1:20
		theta = 0.2 * 25 * (j/20) * 10^i_min;
		E = energy(theta, Cv, Ys, m);
		if (E <= e_min)
			theta_m = theta;
			e_min = E;
		end
	end

	theta = theta_m;

	C = exp(-theta * Cv);
	invC = pinv(C);
	beta = (ones(1,m) * invC * Ys) / (ones(1,m) * invC * ones(m,1));
	s = ((Ys - beta)' * invC * (Ys - beta)) / m;
	lambda = (Ys - beta)' * invC;


function e = energy(theta, Cv, Y, m)
	C = exp(-theta * Cv);
	if (abs(det(C)) > 1e-60)
		invC = pinv(C);
		beta = (ones(1,m) * invC * Y) / (ones(1,m) * invC * ones(m,1));
		s = ((Y - beta)' * invC * (Y - beta)) / m;
		e = m * log(s) + log(det(C));
	else
		e = Inf;
	end
